@@ -1,21 +1,18 @@
 from django import forms
-from django.core.exceptions import FieldDoesNotExist
-from django.forms.utils import pretty_name
 from django.views.generic.base import TemplateView
 from django.urls import reverse
 
-from pyston.filters.default_filters import OPERATORS
+from pyston.filters.filters import OPERATORS
 
 from is_core.auth.views import FieldPermissionViewMixin
 from is_core.config import settings
-from is_core.filters import UIFilterMixin, FilterChoiceIterator
-from is_core.generic_views import DefaultModelCoreViewMixin
-from is_core.rest.datastructures import ModelFlatRESTFields, ModelRESTFieldset
+from is_core.generic_views.base import DefaultModelCoreViewMixin
+from is_core.rest.filters import UIFilterMixin, FilterChoiceIterator
+from is_core.rest.datastructures import ModelFlatRestFields, ModelRestFieldset
 from is_core.utils import (
     pretty_class_name, get_export_types_with_content_type, LOOKUP_SEP, get_field_label_from_path
 )
 
-from chamber.utils import get_class_method
 from chamber.utils.http import query_string_from_dict
 
 from pyston.filters.exceptions import FilterIdentifierError
@@ -39,7 +36,7 @@ class Header:
         return self.text
 
 
-class TableViewMixin(FieldPermissionViewMixin):
+class BaseModelTableViewMixin(FieldPermissionViewMixin):
 
     fields = None
     export_fields = None
@@ -53,21 +50,23 @@ class TableViewMixin(FieldPermissionViewMixin):
     enable_columns_manager = False
     field_labels = None
 
-    @property
-    def list_verbose_name(self):
-        return self.model._ui_meta.list_verbose_name
+    title = None
+    list_verbose_name = None
+    verbose_name = None
+    verbose_name_plural = None
+    model_name = None
 
-    @property
-    def verbose_name(self):
-        return self.model._meta.verbose_name
+    def get_list_verbose_name(self):
+        return self.list_verbose_name
 
-    @property
-    def verbose_name_plural(self):
-        return self.model._meta.verbose_name_plural
+    def get_verbose_name(self):
+        return self.verbose_name
 
-    @property
-    def model_name(self):
-        return str(self.model._meta.model_name)
+    def get_verbose_name_plural(self):
+        return self.verbose_name_plural
+
+    def get_model_name(self):
+        return self.model_name
 
     def _get_allowed_fields(self):
         return [
@@ -84,42 +83,20 @@ class TableViewMixin(FieldPermissionViewMixin):
             field for field in self._get_export_fields() if field not in self._get_disallowed_fields_from_permissions()
         ]
 
-    def get_title(self):
-        return self.list_verbose_name % {
-            'verbose_name': self.verbose_name,
-            'verbose_name_plural': self.verbose_name_plural
-        }
-
     def _get_field_filter_widget(self, filter_obj, full_field_name, field):
-        if filter_obj.choices:
-            return forms.Select(choices=FilterChoiceIterator(filter_obj.choices))
-        else:
-            formfield = field.formfield() if hasattr(field, 'formfield') else None
-            if formfield:
-                widget = formfield.widget
-                if hasattr(widget, 'choices'):
-                    widget.choices = FilterChoiceIterator(widget.choices, field)
-
-                if not isinstance(widget, forms.widgets.Textarea):
-                    return widget
-
-            return forms.TextInput()
+        return forms.TextInput()
 
     def _get_method_filter_widget(self, filter_obj, full_field_name, method):
-        if filter_obj.choices:
-            return forms.Select(choices=FilterChoiceIterator(filter_obj.choices))
-        else:
-            return forms.TextInput()
+        return forms.TextInput()
 
     def _get_resource_filter_widget(self, filter_obj, full_field_name):
-        if filter_obj.choices:
-            return forms.Select(choices=FilterChoiceIterator(filter_obj.choices))
-        else:
-            return forms.TextInput()
+        return forms.TextInput()
 
     def _get_filter_widget(self, filter_obj, full_field_name):
         if isinstance(filter_obj, UIFilterMixin):
             return filter_obj.get_widget(self.request)
+        elif filter_obj.choices:
+            return forms.Select(choices=FilterChoiceIterator(filter_obj.choices))
         elif filter_obj.field:
             return self._get_field_filter_widget(filter_obj, full_field_name, filter_obj.field)
         elif filter_obj.method:
@@ -141,23 +118,23 @@ class TableViewMixin(FieldPermissionViewMixin):
         resource = self.get_resource()
         if resource:
             try:
-                filter_obj = resource.filter_manager.get_filter(
+                return resource.filter_manager.get_filter(
                     field_name.split(LOOKUP_SEP), resource, self.request
-                )
-                if (not filter_obj.identifiers_suffix and filter_obj.field and filter_obj.field.is_relation and
-                        filter_obj.field.related_model and
-                        filter_obj.field.related_model._ui_meta.default_ui_filter_by):
-                    return self._get_filter('{}__{}'.format(
-                        field_name, filter_obj.field.related_model._ui_meta.default_ui_filter_by)
-                    )
-                widget = self._get_filter_widget(filter_obj, field_name)
-                operator = self._get_filter_operator_string(filter_obj, widget)
-                filter_term = '{}__{}'.format(field_name, operator)
-                name = 'filter__{}'.format(filter_term)
-                return widget.render(name, None, attrs={'data-filter': filter_term})
+                ) if resource else None
             except FilterIdentifierError:
                 pass
-        return ''
+        return None
+
+    def _render_filter(self, field_name):
+        filter_obj = self._get_filter(field_name)
+        if filter_obj:
+            widget = self._get_filter_widget(filter_obj, field_name)
+            operator = self._get_filter_operator_string(filter_obj, widget)
+            filter_term = '{}__{}'.format(field_name, operator)
+            name = 'filter__{}'.format(filter_term)
+            return widget.render(name, None, attrs={'data-filter': filter_term})
+        else:
+            return ''
 
     def _get_header_order_by(self, field_name):
         resource = self.get_resource()
@@ -190,7 +167,7 @@ class TableViewMixin(FieldPermissionViewMixin):
     def _get_header(self, field_name):
         return Header(
             field_name, self._get_header_label(field_name), self._get_header_order_by(field_name),
-            self._get_filter(field_name)
+            self._render_filter(field_name)
         )
 
     def _get_fields(self):
@@ -206,7 +183,7 @@ class TableViewMixin(FieldPermissionViewMixin):
         return settings.LIST_PER_PAGE if self.list_per_page is None else self.list_per_page
 
     def _generate_rest_fieldset(self):
-        return ModelRESTFieldset.create_from_flat_list(
+        return ModelRestFieldset.create_from_flat_list(
             list(self._get_allowed_extra_fields()) + list(self._get_allowed_fields()),
             self.model
         )
@@ -249,14 +226,12 @@ class TableViewMixin(FieldPermissionViewMixin):
         return self.menu_group_pattern_name
 
     def get_context_data(self, **kwargs):
-        context_data = super(TableViewMixin, self).get_context_data(**kwargs)
-
-        module_name = self.model_name
+        context_data = super().get_context_data(**kwargs)
 
         context_data.update({
             'headers': self._get_headers(),
             'api_url': self._get_api_url(),
-            'module_name': module_name,
+            'module_name': self.get_model_name(),
             'fields': self._get_allowed_fields(),
             'rest_fieldset': self._generate_rest_fieldset(),
             'query_string_filter': self._get_query_string_filter(),
@@ -279,7 +254,46 @@ class TableViewMixin(FieldPermissionViewMixin):
         return get_resource_or_none(self.request, model or self.model)
 
 
-class TableView(DefaultModelCoreViewMixin, TableViewMixin, TemplateView):
+class DjangoTableViewMixin(BaseModelTableViewMixin):
+
+    def _get_field_filter_widget(self, filter_obj, full_field_name, field):
+        formfield = field.formfield() if hasattr(field, 'formfield') else None
+        if formfield:
+            widget = formfield.widget
+            if hasattr(widget, 'choices'):
+                widget.choices = FilterChoiceIterator(widget.choices, field)
+
+            if not isinstance(widget, forms.widgets.Textarea):
+                return widget
+
+        return forms.TextInput()
+
+    def get_model_name(self):
+        return str(self.model._meta.model_name)
+
+    def _get_filter(self, field_name):
+        filter = super()._get_filter(field_name)
+        if (filter and not filter.identifiers_suffix and filter.field and filter.field.is_relation and
+                filter.field.related_model and filter.field.related_model._ui_meta.default_ui_filter_by):
+            return self._get_filter(LOOKUP_SEP.join((
+                field_name, filter.field.related_model._ui_meta.default_ui_filter_by
+            )))
+        return filter
+
+    def get_list_verbose_name(self):
+        list_verbose_name = super().get_list_verbose_name()
+        return self.model._ui_meta.list_verbose_name if list_verbose_name is None else list_verbose_name
+
+    def get_verbose_name(self):
+        verbose_name = super().get_verbose_name()
+        return self.model._meta.verbose_name if verbose_name is None else verbose_name
+
+    def get_verbose_name_plural(self):
+        verbose_name_plural = super().get_verbose_name_plural()
+        return self.model._meta.verbose_name_plural if verbose_name_plural is None else verbose_name_plural
+
+
+class BaseModelTableView(DefaultModelCoreViewMixin, BaseModelTableViewMixin, TemplateView):
 
     template_name = 'is_core/generic_views/table.html'
     view_type = 'list'
@@ -287,32 +301,25 @@ class TableView(DefaultModelCoreViewMixin, TableViewMixin, TemplateView):
     add_button_verbose_name = None
 
     def get_title(self):
-        return self.list_verbose_name % {
-            'verbose_name': self.verbose_name,
-            'verbose_name_plural': self.verbose_name_plural
+        return self.get_list_verbose_name() % {
+            'verbose_name': self.get_verbose_name(),
+            'verbose_name_plural': self.get_verbose_name_plural()
         }
 
-    @property
-    def add_button_verbose_name(self):
-        return self.model._ui_meta.add_button_verbose_name
-
-    def _get_add_button_verbose_name(self):
-        return self.add_button_verbose_name % {
-            'verbose_name': self.verbose_name,
-            'verbose_name_plural': self.verbose_name_plural
-        }
+    def get_add_button_verbose_name(self):
+        return self.add_button_verbose_name
 
     def _get_field_labels(self):
-        return self.field_labels if self.field_labels is not None else self.core.get_ui_field_labels(self.request)
+        return self.field_labels if self.field_labels is not None else self.core.get_field_labels(self.request)
 
     def _get_fields(self):
-        return self.core.get_ui_list_fields(self.request) if self.fields is None else self.fields
+        return self.core.get_list_fields(self.request) if self.fields is None else self.fields
 
     def _get_export_fields(self):
-        return self.core.get_ui_list_export_fields(self.request) if self.export_fields is None else self.export_fields
+        return self.core.get_export_fields(self.request) if self.export_fields is None else self.export_fields
 
     def _get_export_types(self):
-        return self.core.get_ui_list_export_types(self.request) if self.export_types is None else self.export_types
+        return self.core.get_export_types(self.request) if self.export_types is None else self.export_types
 
     def _get_list_per_page(self):
         list_per_page = self.core.get_list_per_page(self.request)
@@ -328,22 +335,25 @@ class TableView(DefaultModelCoreViewMixin, TableViewMixin, TemplateView):
         return self.core.get_add_url(self.request)
 
     def _generate_rest_export_fieldset(self):
-        return ModelFlatRESTFields.create_from_flat_list(
+        return ModelFlatRestFields.create_from_flat_list(
             list(self._get_allowed_export_fields()), self.model
         )
 
     def is_bulk_change_enabled(self):
         return (
-            hasattr(self.core, 'get_ui_bulk_change_url_name') and self.core.get_ui_bulk_change_url_name() and
-            self.core.ui_patterns.get(self.core.get_ui_bulk_change_url_name()).has_permission('get', self.request)
+            hasattr(self.core, 'is_bulk_change_enabled') and self.core.is_bulk_change_enabled() and
+            self.core.ui_patterns.get(self.core.get_bulk_change_url_name()).has_permission('get', self.request)
         )
 
     def get_context_data(self, **kwargs):
-        context_data = super(TableView, self).get_context_data(**kwargs)
+        context_data = super().get_context_data(**kwargs)
         context_data.update({
             'add_url': self._get_add_url(),
             'view_type': self.view_type,
-            'add_button_value': self._get_add_button_verbose_name(),
+            'add_button_value': self.get_add_button_verbose_name() % {
+                'verbose_name': self.get_verbose_name(),
+                'verbose_name_plural': self.get_verbose_name_plural()
+            },
             'enable_bulk_change': self.is_bulk_change_enabled(),
             'bulk_change_snippet_name': self.get_bulk_change_snippet_name(),
             'bulk_change_form_url': self.get_bulk_change_form_url(),
@@ -359,9 +369,19 @@ class TableView(DefaultModelCoreViewMixin, TableViewMixin, TemplateView):
         return '-'.join(('default', self.core.menu_group, 'form'))
 
     def get_bulk_change_form_url(self):
-        return (reverse(
-            ''.join(('IS:', self.core.get_ui_bulk_change_url_name(), '-', self.core.menu_group)))
-            if self.is_bulk_change_enabled() else None)
+        return (
+            reverse(''.join(('IS:', self.core.get_bulk_change_url_name(), '-', self.core.menu_group)))
+            if self.is_bulk_change_enabled() else None
+        )
 
     def _get_menu_group_pattern_name(self):
         return self.core.get_menu_group_pattern_name()
+
+
+class DjangoTableView(DjangoTableViewMixin, BaseModelTableView):
+
+    def get_add_button_verbose_name(self):
+        return (
+            self.model._ui_meta.add_button_verbose_name
+            if self.add_button_verbose_name is None else self.add_button_verbose_name
+        )
