@@ -9,9 +9,6 @@ from copy import deepcopy
 from collections import OrderedDict
 
 from django.utils.translation import ugettext_lazy as _
-from django.http.response import Http404
-from django.core.exceptions import ValidationError
-from django.forms.models import _get_foreign_key
 from django.utils.functional import cached_property
 from django.urls import reverse
 
@@ -19,12 +16,14 @@ import import_string
 
 from is_core.auth.permissions import FieldsSetPermission
 from is_core.config import settings
-from is_core.actions import WebAction, ConfirmRESTAction
-from is_core.generic_views.form_views import AddModelFormView, DetailModelFormView, BulkChangeFormView
-from is_core.generic_views.table_views import TableView
-from is_core.rest.resource import RESTModelResource
-from is_core.rest.paginators import OffsetBasedPaginator
-from is_core.patterns import UIPattern, RESTPattern, DoubleRESTPattern, HiddenRESTPattern
+from is_core.actions import WebAction, ConfirmRestAction
+from is_core.generic_views.form_views import BulkChangeFormView
+from is_core.generic_views.detail_views import DjangoDetailFormView
+from is_core.generic_views.add_views import DjangoAddFormView
+from is_core.generic_views.table_views import DjangoTableView
+from is_core.rest.resource import DjangoCoreResource
+from is_core.rest.paginators import DjangoOffsetBasedPaginator
+from is_core.patterns import UiPattern, RestPattern, DoubleRestPattern
 from is_core.utils import flatten_fieldsets,  GetMethodFieldMixin, get_model_name, PK_PATTERN
 from is_core.menu import LinkMenuItem
 from is_core.loading import register_core
@@ -35,14 +34,14 @@ from is_core.utils.decorators import short_description
 from .auth.permissions import PermissionsSet, IsAdminUser
 
 
-class ISCoreBase(type):
+class CoreBase(type):
     """Metaclass for IS core classes. Its main purpose is automatic registration cores to your application."""
 
     def __new__(cls, *args, **kwargs):
         name, _, attrs = args
 
         abstract = attrs.pop('abstract', False)
-        super_new = super(ISCoreBase, cls).__new__
+        super_new = super(CoreBase, cls).__new__
         new_class = super_new(cls, *args, **kwargs)
         model_module = sys.modules[new_class.__module__]
         app_label = model_module.__name__.split('.')[-2]
@@ -52,7 +51,7 @@ class ISCoreBase(type):
         return new_class
 
 
-class ISCore(metaclass=ISCoreBase):
+class Core(metaclass=CoreBase):
     """
     Parent of all IS cores. Contains common methods for all cores.
     This class is abstract.
@@ -61,21 +60,23 @@ class ISCore(metaclass=ISCoreBase):
     abstract = True
     register = True
 
-    verbose_name = None
-    verbose_name_plural = None
+    list_actions = ()
+
     menu_group = None
 
     can_create = False
     can_read = True
     can_update = False
     can_delete = False
-    can_delete = False
 
     default_permission = IsAdminUser()
 
     permission = None
     field_permissions = FieldsSetPermission()
 
+    verbose_name = None
+    verbose_name_plural = None
+
     def __init__(self, site_name, menu_parent_groups):
         self.site_name = site_name
         self.menu_parent_groups = menu_parent_groups
@@ -89,7 +90,6 @@ class ISCore(metaclass=ISCoreBase):
 
     def _generate_permission_set(self):
         permission_dict = {}
-      
         for permission_name in ('create', 'read', 'update', 'delete'):
             if getattr(self, 'can_{}'.format(permission_name)):
                 permission_dict[permission_name] = self._get_default_permission(permission_name)
@@ -122,12 +122,14 @@ class ISCore(metaclass=ISCoreBase):
     def get_menu_group_pattern_name(self):
         return '-'.join(self.get_menu_groups())
 
+    def get_verbose_name(self):
+        return self.verbose_name
 
-class ModelISCore(GetMethodFieldMixin, ISCore):
-    """
-    Parent of REST and UI cores that works as controller to specific model.
-    This class is abstract.
-    """
+    def get_verbose_name_plural(self):
+        return self.verbose_name_plural
+
+
+class ModelCore(GetMethodFieldMixin, Core):
 
     abstract = True
 
@@ -135,29 +137,62 @@ class ModelISCore(GetMethodFieldMixin, ISCore):
 
     register_model = True
 
-    # form params
-    form_fields = None
-    form_exclude = ()
-    form_class = SmartModelForm
-    form_edit_class = None
-    form_add_class = None
+    fields = ()
+    list_fields = None
 
     default_ordering = None
 
     field_labels = {}
 
+    can_read = True
+    can_create = False
+    can_update = False
+    can_delete = False
+
+    menu_group = None
+
+    def get_fields(self, request, obj=None):
+        return self.fields
+
+    def get_field_labels(self, request):
+        return self.field_labels
+
+    def get_default_ordering(self):
+        return self.default_ordering
+
+    def get_queryset(self, request):
+        raise NotImplementedError
+
+    def get_list_actions(self, request, obj):
+        return list(self.list_actions)
+
+    def get_default_action(self, request, obj):
+        return None
+
+
+class DjangoCore(ModelCore):
+    """
+    Parent of REST and UI cores that works as controller to specific model.
+    This class is abstract.
+    """
+
+    abstract = True
+
+    # form params
+    form_class = SmartModelForm
+    form_edit_class = None
+    form_add_class = None
+
     can_read = True
     can_create = True
     can_update = True
     can_delete = True
 
-    @property
-    def verbose_name(self):
-        return self.model._meta.verbose_name
+    def get_verbose_name(self):
+        return self.model._meta.verbose_name if self.verbose_name is None else self.verbose_name
 
-    @property
-    def verbose_name_plural(self):
-        return self.model._meta.verbose_name_plural
+    def get_verbose_name_plural(self):
+        return self.model._meta.verbose_name_plural if self.verbose_name_plural is None else self.verbose_name_plural
 
     def get_form_class(self, request, obj=None):
         return self.form_class
@@ -168,15 +203,6 @@ class ModelISCore(GetMethodFieldMixin, ISCore):
     def get_form_add_class(self, request, obj=None):
         return self.form_add_class or self.get_form_class(request, obj)
 
-    def get_form_fields(self, request, obj=None):
-        return self.form_fields
-
-    def get_form_exclude(self, request, obj=None):
-        return self.form_exclude
-
-    def get_field_labels(self, request):
-        return self.field_labels
-
     def pre_save_model(self, request, obj, form, change):
         pass
 
@@ -202,20 +228,14 @@ class ModelISCore(GetMethodFieldMixin, ISCore):
     def get_default_ordering(self):
         return self.default_ordering if self.default_ordering is not None else (self.model._meta.ordering or ('pk',))
 
-    def get_queryset(self, request):
-        return self.model._default_manager.get_queryset().order_by(*self.get_default_ordering())
-
     def preload_queryset(self, request, qs):
         return qs
 
-    def get_list_actions(self, request, obj):
-        return list(self.list_actions)
-
-    def get_default_action(self, request, obj):
-        return None
+    def get_queryset(self, request):
+        return self.model._default_manager.get_queryset().order_by(*self.get_default_ordering())
 
 
-class UIISCore(ISCore):
+class UiCore(Core):
     """Main core for UI views."""
 
     abstract = True
@@ -223,7 +243,7 @@ class UIISCore(ISCore):
     show_in_menu = True
     menu_url_name = None
     view_classes = ()
-    default_ui_pattern_class = UIPattern
+    default_ui_pattern_class = UiPattern
 
     def init_ui_request(self, request):
         self.init_request(request)
@@ -271,18 +291,18 @@ class UIISCore(ISCore):
 
     def get_menu_item(self, request, active_group, item_class=LinkMenuItem):
         if self.get_show_in_menu(request):
-            return item_class(self.verbose_name_plural, self.menu_url(request),
+            return item_class(self.get_verbose_name_plural(), self.menu_url(request),
                               self.menu_group, self.is_active_menu_item(request, active_group))
 
     def menu_url(self, request):
         return self.ui_patterns.get(self.menu_url_name).get_url_string(request)
 
 
-class RESTISCore(ISCore):
+class RestCore(Core):
     """Main core for REST views."""
 
     rest_classes = ()
-    default_rest_pattern_class = RESTPattern
+    default_rest_pattern_class = RestPattern
     abstract = True
 
     api_url_name = None
@@ -303,10 +323,10 @@ class RESTISCore(ISCore):
             name, rest_vals = (rest_class_definition[0], rest_class_definition[1:])
             if name not in rest_patterns:
                 if len(rest_vals) == 3:
-                    pattern, rest, RESTPatternClass = rest_vals
+                    pattern, rest, RestPatternClass = rest_vals
                 else:
                     pattern, rest = rest_vals
-                    RESTPatternClass = self.default_rest_pattern_class
+                    RestPatternClass = self.default_rest_pattern_class
 
                 if isinstance(rest, str):
                     rest = getattr(self, rest)()
@@ -315,41 +335,44 @@ class RESTISCore(ISCore):
                 group_pattern_name = self.get_menu_group_pattern_name()
                 if group_pattern_name:
                     pattern_names += [self.get_menu_group_pattern_name()]
-                rest_patterns[name] = RESTPatternClass('-'.join(pattern_names), self.site_name, pattern, rest, self)
+                rest_patterns[name] = RestPatternClass('-'.join(pattern_names), self.site_name, pattern, rest, self)
         return rest_patterns
 
     def get_urls(self):
         return self.get_urlpatterns(self.rest_patterns)
 
-    def get_api_url(self, request):
-        return reverse(self.get_api_url_name())
-
     def get_api_url_name(self):
         return self.api_url_name or '{}:api-{}'.format(self.site_name, self.get_menu_group_pattern_name())
 
+    def get_api_url(self, request):
+        return reverse(self.get_api_url_name())
+
     def get_api_detail_url_name(self):
         return self.api_url_name or '{}:api-resource-{}'.format(self.site_name, self.get_menu_group_pattern_name())
 
+    def get_api_detail_url(self, request, obj):
+        return reverse(self.get_api_detail_url_name(), kwargs={'pk': obj.pk})
 
-class UIRESTISCoreMixin:
+
+class UiRestCoreMixin:
     """Helper that joins urls generated wit REST core and UI core."""
 
     def get_urls(self):
         return self.get_urlpatterns(self.rest_patterns) + self.get_urlpatterns(self.ui_patterns)
 
 
-class UIRESTISCore(UIRESTISCoreMixin, UIISCore, RESTISCore):
+class UiRestCore(UiRestCoreMixin, UiCore, RestCore):
     """UI REST Core, its main purpose is create custom REST resources and UI views."""
 
     abstract = True
 
 
-class HomeUIISCore(UIISCore):
+class HomeUiCore(UiCore):
 
     menu_url_name = 'index'
     verbose_name_plural = _('Home')
     menu_group = 'home'
-    abstract = settings.HOME_CORE != 'is_core.main.HomeUIISCore'
+    abstract = settings.HOME_CORE != 'is_core.main.HomeUiCore'
 
     def get_view_classes(self):
         view_classes = super().get_view_classes()
@@ -363,8 +386,7 @@ class HomeUIISCore(UIISCore):
         return ''
 
 
-class UIModelISCore(ModelISCore, UIISCore):
-    """Main core controller for specific model that provides UI views for model management (add, edit, list)."""
+class ModelUiCore(ModelCore, UiCore):
 
     abstract = True
 
@@ -372,30 +394,22 @@ class UIModelISCore(ModelISCore, UIISCore):
 
     # list view params
     list_per_page = None
-    detail_export_types = settings.EXPORT_TYPES
     default_list_filter = {}
 
     # add/edit view params
-    form_fieldsets = None
-    form_readonly_fields = ()
-    form_inline_views = None
+    fieldsets = None
+    readonly_fields = None
+    inline_views = None
 
-    ui_form_add_class = None
-    ui_form_edit_class = None
-    ui_list_fields = ('_obj_name',)
-    ui_export_fields = ()
-    ui_list_export_fields = None
-    ui_detail_export_fields = None
-    ui_export_types = settings.EXPORT_TYPES
-    ui_list_export_types = None
-    ui_detail_export_types = None
-    ui_add_view = AddModelFormView
-    ui_detail_view = DetailModelFormView
-    ui_list_view = TableView
-    ui_field_labels = None
+    list_fields = None
+    export_fields = None
+    list_view_export_fields = None
+    detail_view_export_fields = None
+    export_types = settings.EXPORT_TYPES
 
-    ui_bulk_change_url_name = None
-    ui_bulk_change_fields = ()
+    ui_add_view = None
+    ui_detail_view = None
+    ui_list_view = None
 
     @cached_property
     def default_model_view_classes(self):
@@ -414,10 +428,7 @@ class UIModelISCore(ModelISCore, UIISCore):
         return default_model_view_classes
 
     def get_view_classes(self):
-        view_classes = super(UIModelISCore, self).get_view_classes() + list(self.default_model_view_classes)
-        if self.is_bulk_change_enabled():
-            view_classes.append((self.get_ui_bulk_change_url_name(), r'bulk-change/?', BulkChangeFormView))
-        return view_classes
+        return super().get_view_classes() + list(self.default_model_view_classes)
 
     def get_ui_add_view(self):
         return self.ui_add_view
@@ -428,32 +439,15 @@ class UIModelISCore(ModelISCore, UIISCore):
     def get_ui_list_view(self):
         return self.ui_list_view
 
-    def get_ui_form_add_class(self, request, obj=None):
-        return self.ui_form_add_class or self.get_form_add_class(request, obj)
+    def get_fieldsets(self, request, obj=None):
+        return self.fieldsets
 
-    def get_ui_form_edit_class(self, request, obj=None):
-        return self.ui_form_edit_class or self.get_form_edit_class(request, obj)
+    def get_fields(self, request, obj=None):
+        fieldsets = self.get_fieldsets(request, obj)
+        return flatten_fieldsets(fieldsets) if fieldsets is not None else self.fields
 
-    def get_form_fieldsets(self, request, obj=None):
-        return self.form_fieldsets
-
-    def get_form_readonly_fields(self, request, obj=None):
-        return self.form_readonly_fields
-
-    def get_ui_form_fields(self, request, obj=None):
-        return self.get_form_fields(request, obj)
-
-    def get_ui_form_exclude(self, request, obj=None):
-        return self.get_form_exclude(request, obj)
-
-    def is_bulk_change_enabled(self):
-        return self.get_ui_bulk_change_url_name() is not None
-
-    def get_ui_bulk_change_url_name(self):
-        return self.ui_bulk_change_url_name
-
-    def get_ui_bulk_change_fields(self, request):
-        return self.ui_bulk_change_fields
+    def get_readonly_fields(self, request, obj=None):
+        return [] if self.readonly_fields is None else self.readonly_fields
 
     def get_urls(self):
         return self.get_urlpatterns(self.ui_patterns)
@@ -464,44 +458,20 @@ class UIModelISCore(ModelISCore, UIISCore):
             self.ui_patterns.get(self.menu_url_name).has_permission('get', request)
         )
 
-    def get_form_inline_views(self, request, obj=None):
-        return self.form_inline_views
+    def get_inline_views(self, request, obj=None):
+        return self.inline_views
 
     def get_default_list_filter(self, request):
         return self.default_list_filter.copy()
 
-    def get_ui_list_fields(self, request):
-        return list(self.ui_list_fields)
+    def get_list_fields(self, request):
+        return list(self.list_fields) if self.list_fields is not None else self.get_fields(request)
 
-    def get_ui_export_fields(self, request):
-        return list(self.ui_export_fields)
+    def get_export_fields(self, request, obj=None):
+        return list(self.export_fields) if self.export_fields is not None else self.get_list_fields(request)
 
-    def get_ui_list_export_fields(self, request):
-        return (
-            list(self.ui_list_export_fields) if self.ui_list_export_fields is not None
-            else self.get_ui_export_fields(request)
-        )
-
-    def get_ui_detail_export_fields(self, request, obj=None):
-        return (
-            list(self.ui_detail_export_fields) if self.ui_detail_export_fields is not None
-            else self.get_ui_export_fields(request)
-        )
-
-    def get_ui_export_types(self, request):
-        return list(self.ui_export_types or ())
-
-    def get_ui_list_export_types(self, request):
-        return (
-            list(self.ui_list_export_types) if self.ui_list_export_types is not None
-            else self.get_ui_export_types(request)
-        )
-
-    def get_ui_detail_export_types(self, request):
-        return (
-            list(self.ui_detail_export_types) if self.ui_detail_export_types is not None
-            else self.get_ui_export_types(request)
-        )
+    def get_export_types(self, request, obj=None):
+        return list(self.export_types or ())
 
     def get_list_per_page(self, request):
         return self.list_per_page
@@ -509,14 +479,22 @@ class UIModelISCore(ModelISCore, UIISCore):
     def get_add_url(self, request):
         return self.ui_patterns.get('add').get_url_string(request) if 'add' in self.ui_patterns else None
 
-    def get_ui_field_labels(self, request):
-        return self.ui_field_labels if self.ui_field_labels is not None else self.get_field_labels(request)
-
-    def get_detail_export_types(self):
-        return self.detail_export_types
+    @short_description(_('object name'))
+    def _obj_name(self, obj):
+        return str(obj)
 
 
-class RESTModelISCore(RESTISCore, ModelISCore):
+class DjangoUiCore(DjangoCore, ModelUiCore):
+    """Main core controller for specific model that provides UI views for model management (add, edit, list)."""
+
+    abstract = True
+
+    ui_add_view = DjangoAddFormView
+    ui_detail_view = DjangoDetailFormView
+    ui_list_view = DjangoTableView
+
+
+class ModelRestCore(RestCore, ModelCore):
     """
     Main core controller for specific model that provides REST resources for model management.
     CRUD (POST, GET, PUT, DELETE).
@@ -530,21 +508,20 @@ class RESTModelISCore(RESTISCore, ModelISCore):
     rest_detailed_fields = None
     rest_general_fields = None
     rest_guest_fields = None
-    rest_default_fields = None
+    rest_default_fields = ('_obj_name',)
     rest_filter_fields = None
+    rest_filter_manager = None
     rest_extra_filter_fields = None
     rest_order_fields = None
     rest_extra_order_fields = None
     rest_register = True
 
-    rest_default_fields_extension = settings.REST_DEFAULT_FIELDS_EXTENSION
-
     rest_form_edit_class = None
     rest_form_add_class = None
 
-    rest_resource_class = RESTModelResource
+    rest_resource_class = None
     rest_field_labels = None
-    rest_paginator = OffsetBasedPaginator()
+    rest_paginator = None
 
     def get_rest_allowed_methods(self):
         rest_allowed_methods = ['options']
@@ -571,29 +548,106 @@ class RESTModelISCore(RESTISCore, ModelISCore):
         return self.rest_form_edit_class or self.get_form_edit_class(request, obj)
 
     def get_rest_form_fields(self, request, obj=None):
-        return self.get_form_fields(request, obj)
+        return self.get_fields(request, obj)
 
     def get_rest_form_exclude(self, request, obj=None):
-        return self.get_form_exclude(request, obj)
-
-    def get_rest_default_fields_extension(self, request, obj=None):
-        return list(self.rest_default_fields_extension)
-
-    def get_rest_extra_fields(self, request, obj=None):
-        return list(
-            self.model._rest_meta.extra_fields if self.rest_extra_fields is None
-            else self.rest_extra_fields
-        )
-
-    def get_rest_default_fields(self, request, obj=None):
-        return list(
-            self.model._rest_meta.default_fields if self.rest_default_fields is None
-            else self.rest_default_fields
-        )
+        return []
 
     def get_rest_fields(self, request, obj=None):
         return self.rest_fields
 
+    def get_rest_extra_fields(self, request, obj=None):
+        return list(self.rest_extra_fields or ())
+
+    def get_rest_detailed_fields(self, request, obj=None):
+        return list(self.rest_detailed_fields or ())
+
+    def get_rest_general_fields(self, request, obj=None):
+        return list(self.rest_general_fields or ())
+
+    def get_rest_guest_fields(self, request, obj=None):
+        return list(self.rest_guest_fields or ())
+
+    def get_rest_default_fields(self, request, obj=None):
+        return list(self.rest_default_fields or ())
+
+    def get_rest_filter_fields(self, request):
+        return self.rest_filter_fields
+
+    def get_rest_extra_filter_fields(self, request):
+        return self.rest_extra_filter_fields
+
+    def get_rest_order_fields(self, request):
+        return list(self.rest_order_fields or ())
+
+    def get_rest_extra_order_fields(self, request):
+        return list(self.rest_extra_order_fields or ())
+
+    def get_rest_class(self):
+        resource_kwargs = {}
+        resource_kwargs['paginator'] = self.rest_paginator
+        if self.rest_filter_manager:
+            resource_kwargs['filter_manager'] = self.rest_filter_manager
+        return modelrest_factory(
+            self.model, self.rest_resource_class,
+            resource_kwargs=resource_kwargs,
+            register=self.rest_register
+        )
+
+    def get_rest_patterns(self):
+        rest_patterns = super().get_rest_patterns()
+        rest_patterns.update(
+            DoubleRestPattern(
+                self.get_rest_class(), self.default_rest_pattern_class, self,
+                self.get_rest_allowed_methods()
+            ).patterns
+        )
+        return rest_patterns
+
+    def get_list_actions(self, request, obj):
+        list_actions = super().get_list_actions(request, obj)
+        api_resource = self.rest_patterns.get('api-resource')
+        if self.can_delete and api_resource.has_permission('delete', request, obj=obj):
+            confirm_dialog = ConfirmRestAction.ConfirmDialog(_('Do you really want to delete "%s"') % obj)
+            list_actions.append(ConfirmRestAction('api-resource-{}'.format(self.get_menu_group_pattern_name()),
+                                                  _('Delete'), 'DELETE', confirm_dialog=confirm_dialog,
+                                                  class_name='delete', success_text=_('Record "%s" was deleted') % obj))
+        return list_actions
+
+    # Resource extra fields
+
+    def _rest_links(self, obj, request):
+        rest_links = {}
+        for pattern in self.rest_patterns.values():
+            if pattern.send_in_rest:
+                url = pattern.get_url_string(request, obj=obj)
+                if url:
+                    allowed_methods = pattern.get_allowed_methods(request, obj)
+                    if allowed_methods:
+                        rest_links[pattern.name] = {
+                            'url': url,
+                            'methods': [method.upper() for method in allowed_methods]
+                        }
+        return rest_links
+
+    def _actions(self, obj, request):
+        return self.get_list_actions(request, obj)
+
+    def _default_action(self, obj, request):
+        return self.get_default_action(request, obj=obj)
+
+
+class DjangoRestCore(ModelRestCore, DjangoCore):
+    """
+    Main core controller for specific model that provides REST resources for model management.
+    CRUD (POST, GET, PUT, DELETE).
+    """
+
+    abstract = True
+
+    rest_resource_class = DjangoCoreResource
+    rest_paginator = DjangoOffsetBasedPaginator()
+
     def get_rest_general_fields(self, request, obj=None):
         return list(
             self.model._rest_meta.general_fields if self.rest_general_fields is None
@@ -636,81 +690,27 @@ class RESTModelISCore(RESTISCore, ModelISCore):
             else self.rest_order_fields
         )
 
-    def get_rest_class(self):
-        resource_kwargs = {}
-        resource_kwargs['paginator'] = self.rest_paginator
-        return modelrest_factory(self.model, self.rest_resource_class, resource_kwargs=resource_kwargs,
-                                 register=self.rest_register)
 
-    def get_rest_patterns(self):
-        rest_patterns = super(RESTModelISCore, self).get_rest_patterns()
-        rest_patterns.update(DoubleRESTPattern(self.get_rest_class(), self.default_rest_pattern_class, self,
-                                               self.get_rest_allowed_methods()).patterns)
-        return rest_patterns
-
-    def get_list_actions(self, request, obj):
-        list_actions = super(RESTModelISCore, self).get_list_actions(request, obj)
-        api_resource = self.rest_patterns.get('api-resource')
-        if self.can_delete and api_resource.has_permission('delete', request, obj=obj):
-            confirm_dialog = ConfirmRESTAction.ConfirmDialog(_('Do you really want to delete "%s"') % obj)
-            list_actions.append(ConfirmRESTAction('api-resource-{}'.format(self.get_menu_group_pattern_name()),
-                                                  _('Delete'), 'DELETE', confirm_dialog=confirm_dialog,
-                                                  class_name='delete', success_text=_('Record "%s" was deleted') % obj))
-        return list_actions
-
-    # Resource extra fields
-
-    def _rest_links(self, obj, request):
-        rest_links = {}
-        for pattern in self.rest_patterns.values():
-            if pattern.send_in_rest:
-                url = pattern.get_url_string(request, obj=obj)
-                if url:
-                    allowed_methods = pattern.get_allowed_methods(request, obj)
-                    if allowed_methods:
-                        rest_links[pattern.name] = {
-                            'url': url,
-                            'methods': [method.upper() for method in allowed_methods]
-                        }
-        return rest_links
-
-    def _actions(self, obj, request):
-        return self.get_list_actions(request, obj)
-
-    def _default_action(self, obj, request):
-        return self.get_default_action(request, obj=obj)
-
-
-class UIRESTModelISCore(UIRESTISCoreMixin, RESTModelISCore, UIModelISCore):
-    """
-    Combination of UI views and REST resources. UI views uses REST resources for printing model data, filtering,
-    paging and so on.
-    """
+class ModelUiRestCore(UiRestCoreMixin, ModelRestCore, ModelUiCore):
 
     abstract = True
-    ui_rest_extra_fields = ('_web_links', '_rest_links', '_default_action', '_actions', '_class_names', '_obj_name')
+
+    default_rest_extra_fields = (
+        '_web_links', '_rest_links', '_default_action', '_actions', '_class_names', '_obj_name'
+    )
+
     rest_obj_class_names = ()
 
     def get_rest_extra_fields(self, request, obj=None):
         return (
-            super(UIRESTModelISCore, self).get_rest_extra_fields(request, obj) +
-            list(self.get_ui_list_fields(request)) +
-            list(self.get_ui_list_export_fields(request)) +
-            list(self.get_ui_detail_export_fields(request)) +
-            list(self.ui_rest_extra_fields)
+            super().get_rest_extra_fields(request, obj) +
+            list(self.get_list_fields(request)) +
+            list(self.get_export_fields(request)) +
+            list(self.default_rest_extra_fields)
         )
 
-    def get_api_detail_url(self, request, obj):
-        return reverse(self.get_api_detail_url_name(), kwargs={'pk': obj.pk})
-
-    def get_rest_form_fields(self, request, obj=None):
-        return flatten_fieldsets(self.get_form_fieldsets(request, obj) or ()) or self.get_form_fields(request, obj)
-
-    def get_rest_form_exclude(self, request, obj=None):
-        return self.get_form_readonly_fields(request, obj) + self.get_form_exclude(request, obj)
-
     def get_list_actions(self, request, obj):
-        list_actions = super(UIRESTModelISCore, self).get_list_actions(request, obj)
+        list_actions = super().get_list_actions(request, obj)
         detail_pattern = self.ui_patterns.get('detail')
         if detail_pattern and detail_pattern.has_permission('get', request, obj=obj):
             return [
@@ -742,9 +742,40 @@ class UIRESTModelISCore(UIRESTISCoreMixin, RESTModelISCore, UIModelISCore):
                     web_links[pattern.name] = url
         return web_links
 
-    @short_description(_('object name'))
-    def _obj_name(self, obj):
-        return str(obj)
-
     def _class_names(self, obj, request):
         return self.get_rest_obj_class_names(request, obj)
+
+
+class DjangoUiRestCore(DjangoRestCore, DjangoUiCore, ModelUiRestCore):
+    """
+    Combination of UI views and REST resources. UI views uses REST resources for printing model data, filtering,
+    paging and so on.
+    """
+
+    abstract = True
+
+    bulk_change_url_name = 'bulk-change'
+    bulk_change_fields = ()
+
+    bulk_change_enabled = False
+
+    def get_rest_form_fields(self, request, obj=None):
+        return list(set(super().get_rest_form_fields(request, obj)) | set(self.get_bulk_change_fields(request)))
+
+    def get_rest_form_exclude(self, request, obj=None):
+        return self.get_readonly_fields(request, obj)
+
+    def get_view_classes(self):
+        view_classes = super().get_view_classes()
+        if self.is_bulk_change_enabled():
+            view_classes.append((self.get_bulk_change_url_name(), r'bulk-change/?', BulkChangeFormView))
+        return view_classes
+
+    def is_bulk_change_enabled(self):
+        return self.bulk_change_enabled
+
+    def get_bulk_change_url_name(self):
+        return self.bulk_change_url_name
+
+    def get_bulk_change_fields(self, request):
+        return self.bulk_change_fields
